package de.tu_darmstadt.tk.SmartHomeNetworkSim;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SimulationManager;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleConnection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleLink;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleProtocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.MainFrame;

/**
 * Main class which initialises and connects the different parts and starts the program
 *
 * @author Andreas T. Meyer-Berg
 */
public class Main {
	/**
	 * Model of the smart home
	 */
	static Model m;
	
	/**
	 * Visualisation of the smart home network
	 */
	static MainFrame v;
	
	/**
	 * Controller of the program 
	 */
	static Controller c;
	
	/**
	 * SimulationManager which runs the simulation
	 */
	static SimulationManager sim;
	
	/**
	 * Starts the program
	 * @param args will be ignored
	 */
	public static void main(String[] args) {
		m = new Model();
		c = new Controller(m);
	    v = new MainFrame(m, c);
	    sim = new SimulationManager(m);
	    initializeTest();
	    sim.simulateTimeIntervall(0, 1000);
	}
	
	/**
	 * Initializes a basic test Network, which contains a few SmartDevices, one Link and one Connection
	 */
	public static void initializeTest(){
		SmartDevice A = null, B = null, C = null;
		for(int i = 0; i<5; i++){
			A = new SmartDevice("SmartTV"+i);		
			A.setX((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
			A.setY((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
			B = new SmartDevice("SmartDoor"+i);
			B.setX((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
			B.setY((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
			C = new SmartDevice("SmartLight"+i);
			C.setX((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
			C.setY((int)(Math.random()*m.getWidth()-2*m.getDevice_visualization_radius())+m.getDevice_visualization_radius());
		
			c.addSmartDevice(A);
			c.addSmartDevice(B);
			c.addSmartDevice(C);
		}
		
		Link link = new SimpleLink("SimpleWifi");
		link.addDevice(A);
		link.addDevice(B);
		link.addDevice(C);
		A.addLink(link);
		B.addLink(link);
		C.addLink(link);
		
		Port a = new Port(A, (short) 1);
		a.setLastTrigger(0);
		a.setTriggerInterval(68);
		a.setStatus(Port.SENDING);
		A.addPort(a);
		Port b = new Port(B, (short) 2);
		b.setTriggerInterval(102);
		b.setStatus(Port.SENDING);
		B.addPort(b);
		
		SimpleConnection s = new SimpleConnection(a, b, link, new SimpleProtocol(a, b));
		s.setPacketLossProbability(0.01);//1% Packet loss probability
		a.setConnection(s);
		b.setConnection(s);
		m.addConnectionNetwork(link);
		link.addConnection(s);
		m.addConnection(s);
	}

}
